@@ -489,7 +489,12 @@ static void tracemark(char *fmt, ...)
 	va_start(ap, fmt);
 	len = vsnprintf(tracebuf, TRACEBUFSIZ, fmt, ap);
 	va_end(ap);
+
+	/* write the tracemark message */
 	write(tracemark_fd, tracebuf, len);
+
+	/* now stop any trace */
+	write(trace_fd, "0\n", 2);
 }
 
 
@@ -535,13 +540,28 @@ static void open_tracemark_fd(void)
 {
 	char path[MAX_PATH];
 
-	if (tracemark_fd >= 0)
-		return;
+	/*
+	 * open the tracemark file if it's not already open
+	 */
+	if (tracemark_fd < 0) {
+		sprintf(path, "%s/%s", fileprefix, "trace_marker");
+		tracemark_fd = open(path, O_WRONLY);
+		if (tracemark_fd < 0) {
+			warn("unable to open trace_marker file: %s\n", path);
+			return;
+		}
+	}
 
-	sprintf(path, "%s/%s", fileprefix, "trace_marker");
-	tracemark_fd = open(path, O_WRONLY);
-	if (tracemark_fd < 0)
-		warn("unable to open trace_marker file: %s\n", path);
+	/*
+	 * if we're not tracing and the tracing_on fd is not open,
+	 * open the tracing_on file so that we can stop the trace
+	 * if we hit a breaktrace threshold
+	 */
+	if (notrace && trace_fd < 0) {
+		sprintf(path, "%s/%s", fileprefix, "tracing_on");
+		if ((trace_fd = open(path, O_WRONLY)) < 0)
+			warn("unable to open tracing_on file: %s\n", path);
+	}
 }
 
 static void debugfs_prepare(void)
@@ -1276,17 +1296,16 @@ static void display_help(int error)
 	       "                           0 = CLOCK_MONOTONIC (default)\n"
 	       "                           1 = CLOCK_REALTIME\n"
 	       "-C       --context         context switch tracing (used with -b)\n"
-	       "-d DIST  --distance=DIST   distance of thread intervals in us default=500\n"
-	       "-D       --duration=t      specify a length for the test run\n"
-	       "                           default is in seconds, but 'm', 'h', or 'd' maybe added\n"
-	       "                           to modify value to minutes, hours or days\n"
+	       "-d DIST  --distance=DIST   distance of thread intervals in us, default=500\n"
+	       "-D       --duration=TIME   specify a length for the test run.\n"
+	       "                           Append 'm', 'h', or 'd' to specify minutes, hours or days.\n"
 	       "	 --latency=PM_QOS  write PM_QOS to /dev/cpu_dma_latency\n"
 	       "-E       --event           event tracing (used with -b)\n"
 	       "-f       --ftrace          function trace (when -b is active)\n"
 	       "-F       --fifo=<path>     create a named pipe at path and write stats to it\n"
 	       "-h       --histogram=US    dump a latency histogram to stdout after the run\n"
-	       "                           (with same priority about many threads)\n"
-	       "                           US is the max time to be be tracked in microseconds\n"
+	       "                           US is the max latency time to be be tracked in microseconds\n"
+	       "			   This option runs all threads at the same priority.\n"
 	       "-H       --histofall=US    same as -h except with an additional summary column\n"
 	       "	 --histfile=<path> dump the latency histogram to <path> instead of stdout\n"
 	       "-i INTV  --interval=INTV   base interval of thread in us default=1000\n"
@@ -1296,7 +1315,8 @@ static void display_help(int error)
 	       "			   This will give you poorer realtime results\n"
 	       "			   but will not drain your battery so quickly\n"
 	       "-m       --mlockall        lock current and future memory allocations\n"
-	       "-M       --refresh_on_max  delay updating the screen until a new max latency is hit\n"
+	       "-M       --refresh_on_max  delay updating the screen until a new max\n"
+	       "			   latency is hit. Userful for low bandwidth.\n"
 	       "-n       --nanosleep       use clock_nanosleep\n"
 	       "	 --notrace	   suppress tracing\n"
 	       "-N       --nsecs           print results in ns instead of us (default us)\n"
@@ -1304,20 +1324,23 @@ static void display_help(int error)
 	       "-O TOPT  --traceopt=TOPT   trace option\n"
 	       "-p PRIO  --priority=PRIO   priority of highest prio thread\n"
 	       "-P       --preemptoff      Preempt off tracing (used with -b)\n"
-	       "-q       --quiet           print only a summary on exit\n"
-	       "	 --priospread       spread priority levels starting at specified value\n"
+	       "	 --policy=NAME     policy of measurement thread, where NAME may be one\n"
+	       "                           of: other, normal, batch, idle, fifo or rr.\n"
+	       "	 --priospread      spread priority levels starting at specified value\n"
+	       "-q       --quiet           print a summary only on exit\n"
 	       "-r       --relative        use relative timer instead of absolute\n"
 	       "-R       --resolution      check clock resolution, calling clock_gettime() many\n"
-	       "                           times.  list of clock_gettime() values will be\n"
+	       "                           times.  List of clock_gettime() values will be\n"
 	       "                           reported with -X\n"
-	       "         --secaligned [USEC] align thread wakeups to the next full second,\n"
+	       "         --secaligned [USEC] align thread wakeups to the next full second\n"
 	       "                           and apply the optional offset\n"
 	       "-s       --system          use sys_nanosleep and sys_setitimer\n"
 	       "-S       --smp             Standard SMP testing: options -a -t -n and\n"
 	       "                           same priority of all threads\n"
-	       "	--spike=trigger	   record all spikes > trigger\n"
-	       "	--spike-nodes	   these are the number of spikes we can record\n"
-	       "			   the default is 1024 if not specified\n"
+	       "	--spike=<trigger>  record all spikes > trigger\n"
+	       "	--spike-nodes=[num of nodes]\n"
+	       "			   These are the maximum number of spikes we can record.\n"
+	       "			   The default is 1024 if not specified\n"
 #ifdef ARCH_HAS_SMI_COUNTER
                "         --smi             Enable SMI counting\n"
 #endif
@@ -1337,9 +1360,7 @@ static void display_help(int error)
 	       "                           format: n:c:v n=tasknum c=count v=value in us\n"
 	       "-w       --wakeup          task wakeup tracing (used with -b)\n"
 	       "-W       --wakeuprt        rt task wakeup tracing (used with -b)\n"
-	       "	 --dbg_cyclictest  print info useful for debugging cyclictest\n"
-	       "	 --policy=POLI     policy of realtime thread, POLI may be fifo(default) or rr\n"
-	       "                           format: --policy=fifo(default) or --policy=rr\n",
+	       "	 --dbg_cyclictest  print info useful for debugging cyclictest\n",
 	       tracers
 		);
 	if (error)
