@@ -1,16 +1,20 @@
 #!/usr/bin/python
 
+# (C) 2015 Clark Williams <williams@redhat.com>
 # (C) 2009 Clark Williams <williams@redhat.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License Version 2
 # as published by the Free Software Foundation.
 
+from __future__ import print_function
+
 import sys
 import os
 import time
 import subprocess
 import errno
+import os.path
 
 version = "0.6"
 debugging = False
@@ -75,15 +79,21 @@ class DebugFS(object):
             val = f.readline()
             f.close()
         else:
-            fd = os.open(path, os.O_RDONLY|os.O_NONBLOCK)
+            f = os.fdopen(os.open(path, os.O_RDONLY|os.O_NONBLOCK), "r")
             try:
-                val = os.read(fd, 256)
+                val = f.readline()
             except OSError as e:
+                print ("errno: %s" % e)
                 if e.errno == errno.EAGAIN:
                     val = None
                 else:
                     raise
-            os.close(fd)
+            except IOError as e:
+                if e.errno == errno.EAGAIN:
+                    val = None
+                else:
+                    raise
+            f.close()
         return val
 
     def putval(self, item, value):
@@ -106,6 +116,14 @@ class Kmod(object):
     ''' class to manage loading and unloading hwlat.ko'''
 
     names = ("hwlat_detector", "smi_detector")
+    def __check_builtin(self):
+        for l in open(os.path.join('/lib/modules', os.uname()[2], 'modules.builtin'), "r"):
+            for m in Kmod.names:
+                if m in l:
+                    debug("found %s as builtin" % m)
+                    return m
+        return None
+
     def __find_modname(self):
         debug("looking for modules")
         path = os.path.join("/lib/modules",
@@ -121,6 +139,15 @@ class Kmod(object):
 
     def __init__(self):
         self.preloaded = False
+        self.builtin = False
+
+        # check for builtin
+        self.modname = self.__check_builtin()
+        if self.modname:
+            self.builtin = True
+            return
+
+        # now look for module
         f = open ('/proc/modules')
         for l in f:
             field = l.split()
@@ -134,6 +161,9 @@ class Kmod(object):
         self.modname = self.__find_modname()
 
     def load(self):
+        if self.builtin:
+            debug("not loading %s (builtin)" % self.modname)
+            return True
         if self.preloaded:
             debug("not loading %s (already loaded)" % self.modname)
             return True
@@ -141,7 +171,7 @@ class Kmod(object):
         return (subprocess.call(cmd) == 0)
 
     def unload(self):
-        if self.preloaded:
+        if self.preloaded or self.builtin:
             debug("Not unloading %s" % self.modname)
             return True
         cmd = ['/sbin/modprobe', '-r', self.modname]
@@ -164,6 +194,19 @@ class Detector(object):
             self.detector = Smi(self.debugfs)
         self.samples = []
         self.testduration = 10 # ten seconds
+        self.have_msr = False
+        self.initsmi = []
+        if os.path.exists('/usr/sbin/rdmsr'):
+            self.have_msr = True
+            self.initsmi = self.getsmicounts()
+
+    def getsmicounts(self):
+        counts = []
+        if self.have_msr:
+            p = subprocess.Popen(['/usr/sbin/rdmsr', '-a', '-d', '0x34'], stdout=subprocess.PIPE)
+            p.wait()
+            counts = [ int(x.strip()) for x in p.stdout.readlines()]
+        return counts
 
     def force_cleanup(self):
         debug("forcing unload of hwlat module")
@@ -260,7 +303,7 @@ class Hwlat(object):
                 while val:
                     val = val.strip()
                     self.samples.append(val)
-                    if watch: print val
+                    if watch: print(val)
                     debug("got a latency sample: %s" % val)
                     val = self.get_sample()
                 time.sleep(0.1)
@@ -356,7 +399,7 @@ class Smi(object):
                 val = val.strip()
                 if int(val) >= threshold:
                     self.samples.append(val)
-                    if watch: print val
+                    if watch: print(val)
                     debug("got a latency sample: %s (threshold: %d)" % (val, self.get("threshold")))
                 time.sleep(0.1)
         except KeyboardInterrupt as e:
@@ -530,6 +573,16 @@ if __name__ == '__main__':
     info("Samples recorded: %d" % len(detect.samples))
     info("Samples exceeding threshold: %d" % exceeding)
 
+    if detect.have_msr:
+        finishsmi = detect.getsmicounts()
+        total_smis = 0
+        for i,count in enumerate(finishsmi):
+            if count > detect.initsmi[i]:
+                smis = count - detect.initsmi[i]
+                total_smis += smis
+                print("%d SMIs occured on cpu %d" % (smis, i))
+        info("SMIs during run: %d" % total_smis)
+
     if reportfile:
         count = 0
         f = open(reportfile, "w")
